@@ -26,7 +26,7 @@
  */
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /CVS/nmap/libpcap-possiblymodified/pcap-linux.c,v 1.4 2002/12/22 20:31:30 fyodor Exp $ (LBL)";
+    "@(#) $Header: /CVS/nmap/libpcap-possiblymodified/pcap-linux.c,v 1.5 2003/09/20 09:03:01 fyodor Exp $ (LBL)";
 #endif
 
 /*
@@ -157,6 +157,16 @@ typedef int		socklen_t;
 #define MSG_TRUNC	0x20
 #endif
 
+#ifndef SOL_PACKET
+/*
+ * This is being compiled on a system that lacks SOL_PACKET; define it
+ * with the value it has in the 2.2 and later kernels, so that we can
+ * set promiscuous mode in the good modern way rather than the old
+ * 2.0-kernel crappy way.
+ */
+#define SOL_PACKET	263
+#endif
+
 #define MAX_LINKHEADER_SIZE	256
 
 /* 
@@ -169,7 +179,7 @@ typedef int		socklen_t;
 /*
  * Prototypes for internal functions
  */
-static void map_arphrd_to_dlt(pcap_t *, int);
+static void map_arphrd_to_dlt(pcap_t *, int, int);
 static int live_open_old(pcap_t *, char *, int, int, char *);
 static int live_open_new(pcap_t *, char *, int, int, char *);
 static int pcap_read_packet(pcap_t *, pcap_handler, u_char *);
@@ -214,6 +224,8 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 {
 	pcap_t		*handle;
 	int		mtu;
+	int		err;
+	int		live_open_ok = 0;
 	struct utsname	utsname;
 
         /* Allocate a handle for this session. */
@@ -265,15 +277,21 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 	 * trying both methods with the newer method preferred.
 	 */
 
-	if (! (live_open_new(handle, device, promisc, to_ms, ebuf) ||
-	       live_open_old(handle, device, promisc, to_ms, ebuf)) )
-	{
+	if ((err = live_open_new(handle, device, promisc, to_ms, ebuf)) == 1)
+		live_open_ok = 1;
+	else if (err == 0) {
+		/* Non-fatal error; try old way */
+		if (live_open_old(handle, device, promisc, to_ms, ebuf))
+			live_open_ok = 1;
+	}
+	if (!live_open_ok) {
 		/* 
 		 * Both methods to open the packet socket failed. Tidy
 		 * up and report our failure (ebuf is expected to be
 		 * set by the functions above). 
 		 */
 
+		if (handle->md.device != NULL)
 		free(handle->md.device);
 		free(handle);
 		return NULL;
@@ -332,7 +350,11 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 		 */
 		mtu = iface_get_mtu(handle->fd, device, ebuf);
 		if (mtu == -1) {
+			if (handle->md.clear_promisc)
+				/* 2.0.x kernel */
+				pcap_close_linux(handle);
 			close(handle->fd);
+			if (handle->md.device != NULL)
 			free(handle->md.device);
 			free(handle);
 			return NULL;
@@ -360,7 +382,11 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 	if (!handle->buffer) {
 	        snprintf(ebuf, PCAP_ERRBUF_SIZE,
 			 "malloc: %s", pcap_strerror(errno));
+		if (handle->md.clear_promisc)
+			/* 2.0.x kernel */
+			pcap_close_linux(handle);
 		close(handle->fd);
+		if (handle->md.device != NULL)
 		free(handle->md.device);
 		free(handle);
 		return NULL;
@@ -746,6 +772,7 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 #ifdef SO_ATTACH_FILTER
 	struct sock_fprog	fcode;
 	int			can_filter_in_kernel;
+	int			err = 0;
 #endif
 
 	if (!handle)
@@ -758,11 +785,9 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 
 	/* Make our private copy of the filter */
 
-	if (install_bpf_program(handle, filter) < 0) {
-		snprintf(handle->errbuf, sizeof(handle->errbuf),
-			 "malloc: %s", pcap_strerror(errno));
+	if (install_bpf_program(handle, filter) < 0)
+		/* install_bpf_program() filled in errbuf */
 		return -1;
-	}
 
 	/* 
 	 * Run user level packet filter by default. Will be overriden if 
@@ -836,12 +861,12 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 	}
 
 	if (can_filter_in_kernel) {
-		if (set_kernel_filter(handle, &fcode) == 0)
+		if ((err = set_kernel_filter(handle, &fcode)) == 0)
 		{
 			/* Installation succeded - using kernel filter. */
 			handle->md.use_bpf = 1;
 		}
-		else
+		else if (err == -1)	/* Non-fatal error */
 		{
 			/* 
 			 * Print a warning if we weren't able to install
@@ -872,6 +897,10 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 	 */
 	if (fcode.filter != NULL)
 		free(fcode.filter);
+
+	if (err == -2)
+		/* Fatal error */
+		return -1;
 #endif /* SO_ATTACH_FILTER */
 
 	return 0;
@@ -889,9 +918,13 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
  *  (If the offset isn't set here, it'll be 0; add code as appropriate
  *  for cases where it shouldn't be 0.)
  *  
+ *  If "cooked_ok" is non-zero, we can use DLT_LINUX_SLL and capture
+ *  in cooked mode; otherwise, we can't use cooked mode, so we have
+ *  to pick some type that works in raw mode, or fail.
+ *  
  *  Sets the link type to -1 if unable to map the type.
  */
-static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
+static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 {
 	switch (arptype) {
 
@@ -931,6 +964,9 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
 		handle->linktype = DLT_ARCNET;
 		break;
 
+#ifndef ARPHRD_FDDI	/* From Linux 2.2.13 */
+#define ARPHRD_FDDI	774
+#endif
 	case ARPHRD_FDDI:
 		handle->linktype = DLT_FDDI;
 		handle->offset = 3;
@@ -973,9 +1009,13 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
 		 * Both of those are a nuisance - and, at least on systems
 		 * that support PF_PACKET sockets, we don't have to put
 		 * up with those nuisances; instead, we can just capture
-		 * in cooked mode.  That's what we'll do.
+		 * in cooked mode.  That's what we'll do, if we can.
+		 * Otherwise, we'll just fail.
 		 */
+		if (cooked_ok)
 		handle->linktype = DLT_LINUX_SLL;
+		else
+			handle->linktype = -1;
 		break;
 
 #ifndef ARPHRD_IEEE80211  /* From Linux 2.4.6 */
@@ -985,6 +1025,13 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
 		handle->linktype = DLT_IEEE802_11;
 		break;
 
+#ifndef ARPHRD_IEEE80211_PRISM  /* From Linux 2.4.18 */
+#define ARPHRD_IEEE80211_PRISM 802
+#endif
+	case ARPHRD_IEEE80211_PRISM:
+		handle->linktype = DLT_PRISM_HEADER;
+		break;
+
 	case ARPHRD_PPP:
 		/*
 		 * Some PPP code in the kernel supplies no link-layer
@@ -998,14 +1045,40 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
 		 * oddball link-layer headers particular packets have).
 		 *
 		 * As such, we just punt, and run all PPP interfaces
-		 * in cooked mode.
+                 * in cooked mode, if we can; otherwise, we just treat
+                 * it as DLT_RAW, for now - if somebody needs to capture,
+                 * on a 2.0[.x] kernel, on PPP devices that supply a
+                 * link-layer header, they'll have to add code here to
+                 * map to the appropriate DLT_ type (possibly adding a
+                 * new DLT_ type, if necessary).
 		 */
-		handle->linktype = DLT_LINUX_SLL;
+                if (cooked_ok)
+		  handle->linktype = DLT_LINUX_SLL;
+		else {
+		  /*
+		   * XXX - handle ISDN types here?  We can't fall
+		   * back on cooked sockets, so we'd have to
+		   * figure out from the device name what type of
+		   * link-layer encapsulation it's using, and map
+		   * that to an appropriate DLT_ value, meaning
+		   * we'd map "isdnN" devices to DLT_RAW (they
+		   * supply raw IP packets with no link-layer
+		   * header) and "isdY" devices to a new DLT_I4L_IP
+		   * type that has only an Ethernet packet type as
+		   * a link-layer header.
+		   *
+		   * But sometimes we seem to get random crap
+		   * in the link-layer header when capturing on
+		   * ISDN devices....
+		   */
+                        handle->linktype = DLT_RAW;
+		}
 		break;
 
 #ifndef ARPHRD_CISCO /* Cisco HDLC - From Linux 2.4.17 include/linux/if_arp.h */
 #define ARPHRD_CISCO 513
 #endif
+
 #ifndef ARPHRD_HDLC /* From Linux 2.4.17 */
 #define ARPHRD_HDLC ARPHRD_CISCO
 #endif
@@ -1018,7 +1091,7 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
 	 * works for CIPE */
 	case ARPHRD_TUNNEL:
 #ifndef ARPHRD_SIT
-#define ARPHRD_SIT 776	/* From Linux 2.2.14 */
+#define ARPHRD_SIT 776	/* From Linux 2.2.13 */
 #endif
 	case ARPHRD_SIT:
 	case ARPHRD_CSLIP:
@@ -1026,6 +1099,10 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype)
 	case ARPHRD_CSLIP6:
 	case ARPHRD_ADAPT:
 	case ARPHRD_SLIP:
+#ifndef ARPHRD_RAWHDLC
+#define ARPHRD_RAWHDLC 518
+#endif
+	case ARPHRD_RAWHDLC:
 		/*
 		 * XXX - should some of those be mapped to DLT_LINUX_SLL
 		 * instead?  Should we just map all of them to DLT_LINUX_SLL?
@@ -1056,6 +1133,8 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 {
 #ifdef HAVE_PF_PACKET_SOCKETS
 	int			sock_fd = -1, device_id, arptype;
+	int			err;
+	int			fatal_err = 0;
 	struct packet_mreq	mr;
 
 	/* One shot loop used for error handling - bail out with break */
@@ -1108,9 +1187,11 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 			handle->md.cooked = 0;
 
 			arptype	= iface_get_arptype(sock_fd, device, ebuf);
-			if (arptype == -1) 
+			if (arptype == -1) {
+				fatal_err = 1;
 				break;
-			map_arphrd_to_dlt(handle, arptype);
+			}
+			map_arphrd_to_dlt(handle, arptype, 1);
 			if (handle->linktype == -1 ||
 			    handle->linktype == DLT_LINUX_SLL ||
 			    (handle->linktype == DLT_EN10MB &&
@@ -1160,8 +1241,11 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 			if (device_id == -1)
 				break;
 
-			if (iface_bind(sock_fd, device_id, ebuf) == -1)
+			if ((err = iface_bind(sock_fd, device_id, ebuf)) < 0) {
+				if (err == -2)
+					fatal_err = 1;
 				break;
+			}
 		} else {
 			/*
 			 * This is cooked mode.
@@ -1182,7 +1266,6 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 
 		/* Select promiscuous mode on/off */
 
-#ifdef SOL_PACKET
 		/* 
 		 * Hmm, how can we set promiscuous mode on all interfaces?
 		 * I am not sure if that is possible at all.
@@ -1201,7 +1284,6 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 				break;
 			}
 		}
-#endif
 
 		/* Save the socket FD in the pcap structure */
 
@@ -1213,6 +1295,10 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 
 	if (sock_fd != -1)
 		close(sock_fd);
+
+	if (fatal_err)
+		return -2;
+	else
 	return 0;
 #else
 	strncpy(ebuf, 
@@ -1251,6 +1337,8 @@ static int
 iface_bind(int fd, int ifindex, char *ebuf)
 {
 	struct sockaddr_ll	sll;
+	int			err;
+	socklen_t		errlen = sizeof(err);
 
 	memset(&sll, 0, sizeof(sll));
 	sll.sll_family		= AF_PACKET;
@@ -1263,6 +1351,20 @@ iface_bind(int fd, int ifindex, char *ebuf)
 		return -1;
 	}
 
+	/* Any pending errors, e.g., network is down? */
+
+	if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &errlen) == -1) {
+		snprintf(ebuf, PCAP_ERRBUF_SIZE,
+			"getsockopt: %s", pcap_strerror(errno));
+		return -2;
+	}
+
+	if (err > 0) { 
+		snprintf(ebuf, PCAP_ERRBUF_SIZE,
+			"bind: %s", pcap_strerror(err)); 
+		return -2;
+	}
+
 	return 0;
 }
 
@@ -1368,8 +1470,10 @@ void	pcap_close_linux( pcap_t *handle )
 			}
 		}
 	}
+
 	if (handle->md.device != NULL)
 		free(handle->md.device);
+		handle->md.device = NULL;
 }
 
 /*
@@ -1410,7 +1514,26 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 		if (iface_bind_old(sock_fd, device, ebuf) == -1)
 			break;
 
-		/* Go to promisc mode */
+		/*
+		 * Try to get the link-layer type.
+		 */
+		arptype = iface_get_arptype(sock_fd, device, ebuf);
+		if (arptype == -1)
+			break;
+
+		/*
+		 * Try to find the DLT_ type corresponding to that
+		 * link-layer type.
+		 */
+		map_arphrd_to_dlt(handle, arptype, 0);
+		if (handle->linktype == -1) {
+			snprintf(ebuf, PCAP_ERRBUF_SIZE,
+				 "unknown arptype %d", arptype);
+			break;
+		}
+
+		/* Go to promisc mode if requested */
+
 		if (promisc) {
 			memset(&ifr, 0, sizeof(ifr));
 			strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
@@ -1444,6 +1567,7 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 							PCAP_ERRBUF_SIZE);
 						break;
 					}
+					did_atexit = 1;
 				}
 
 				ifr.ifr_flags |= IFF_PROMISC;
@@ -1464,12 +1588,6 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 			}
 		}
 
-		/* All done - fill in the pcap handle */
-
-		arptype = iface_get_arptype(sock_fd, device, ebuf);
-		if (arptype == -1)
-			break;
-
 		/* Save the socket FD in the pcap structure */
 
 		handle->fd 	 = sock_fd;
@@ -1480,29 +1598,12 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 		 */
 		handle->offset	 = 0;
 
-		/*
-		 * XXX - handle ISDN types here?  We can't fall back on
-		 * cooked sockets, so we'd have to figure out from the
-		 * device name what type of link-layer encapsulation
-		 * it's using, and map that to an appropriate DLT_
-		 * value, meaning we'd map "isdnN" devices to DLT_RAW
-		 * (they supply raw IP packets with no link-layer
-		 * header) and "isdY" devices to a new DLT_I4L_IP
-		 * type that has only an Ethernet packet type as
-		 * a link-layer header.
-		 */
-		map_arphrd_to_dlt(handle, arptype);
-		if (handle->linktype == -1 ||
-		    handle->linktype == DLT_LINUX_SLL) {
-			snprintf(ebuf, PCAP_ERRBUF_SIZE,
-				 "interface type of %s not supported", device);
-			break;
-		}
-
 		return 1;
 
 	} while (0);
 
+	if (handle->md.clear_promisc)
+		pcap_close_linux(handle);
 	if (sock_fd != -1)
 		close(sock_fd);
 	return 0;
@@ -1516,6 +1617,8 @@ static int
 iface_bind_old(int fd, const char *device, char *ebuf)
 {
 	struct sockaddr	saddr;
+	int		err;
+	socklen_t	errlen = sizeof(err);
 
 	memset(&saddr, 0, sizeof(saddr));
 	strncpy(saddr.sa_data, device, sizeof(saddr.sa_data));
@@ -1525,6 +1628,20 @@ iface_bind_old(int fd, const char *device, char *ebuf)
 		return -1;
 	}
 
+	/* Any pending errors, e.g., network is down? */
+
+	if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &errlen) == -1) {
+		snprintf(ebuf, PCAP_ERRBUF_SIZE,
+			"getsockopt: %s", pcap_strerror(errno));
+		return -1;
+	}
+
+	if (err > 0) { 
+		snprintf(ebuf, PCAP_ERRBUF_SIZE,
+			"bind: %s", pcap_strerror(err)); 
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -1750,7 +1867,7 @@ set_kernel_filter(pcap_t *handle, struct sock_fprog *fcode)
 		/*
 		 * Save the socket's current mode, and put it in
 		 * non-blocking mode; we drain it by reading packets
-		 * until we get an error (which we assume is a
+		 * until we get an error (which is normally a
 		 * "nothing more to be read" error).
 		 */
 		save_mode = fcntl(handle->fd, F_GETFL, 0);
@@ -1759,7 +1876,15 @@ set_kernel_filter(pcap_t *handle, struct sock_fprog *fcode)
 			while (recv(handle->fd, &drain, sizeof drain,
 			       MSG_TRUNC) >= 0)
 				;
+			save_errno = errno;
 			fcntl(handle->fd, F_SETFL, save_mode);
+			if (save_errno != EAGAIN) {
+				/* Fatal error */
+				reset_kernel_filter(handle);
+				snprintf(handle->errbuf, sizeof(handle->errbuf),
+				 "recv: %s", pcap_strerror(save_errno));
+				return -2;
+			}
 		}
 	}
 
