@@ -82,7 +82,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_str.c,v 1.7 2003/09/11 02:12:57 fyodor Exp $ */
+/* $Id: nbase_str.c,v 1.8 2003/12/02 01:09:39 fyodor Exp $ */
 
 #include "nbase.h"
 #include <string.h>
@@ -129,3 +129,24 @@ int Strncpy(char *dest, const char *src, size_t n) {
   dest[n-1] = '\0';
   return -1;
 }
+
+/* Trivial function that returns nonzero if all characters in str of length strlength are
+   printable (as defined by isprint()) */
+int stringisprintable(const char *str, int strlength) {
+  int i;
+  for(i=0; i < strlength; i++)
+    if (!isprint((unsigned char) str[i]))
+      return 0;
+
+  return 1;
+}
+
+/* Convert non-printable characters to replchar in the string */
+void replacenonprintable(char *str, int strlength, char replchar) {
+  int i;
+  for(i=0; i < strlength; i++)
+    if (!isprint( (unsigned char) str[i]))
+      str[i] = replchar;
+
+  return;
+}
