@@ -83,7 +83,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc,v 1.35 2003/09/15 07:28:45 fyodor Exp $ */
+/* $Id: nmap.cc,v 1.38 2003/12/02 01:09:39 fyodor Exp $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -1340,7 +1340,7 @@ void printusage(char *name, int rc) {
 	 "* -sU UDP port scan\n"
 	 "  -sP ping scan (Find any reachable machines)\n"
 	 "* -sF,-sX,-sN Stealth FIN, Xmas, or Null scan (experts only)\n"
-         "  -sV Version scan probes open ports determining service and app names/versions"
+         "  -sV Version scan probes open ports determining service & app names/versions\n"
 	 "  -sR/-I RPC/Identd scan (use with other scan types)\n"
 	 "Some Common Options (none are required, most can be combined):\n"
 	 "* -O Use TCP/IP fingerprinting to guess remote operating system\n"
@@ -1713,6 +1713,7 @@ int getidentinfoz(struct in_addr target, u16 localport, u16 remoteport,
   char response[1024];
   char *p,*q;
   char  *os;
+  int slen = 0;
 
   if (ownersz == 0) return 0;
   owner[0] = '\0';
@@ -1763,6 +1764,8 @@ int getidentinfoz(struct in_addr target, u16 localport, u16 remoteport,
 	    if ((q = strchr(p, '\r'))) *q = '\0';
 	    if ((q = strchr(p, '\n'))) *q = '\0';
 	    Strncpy(owner, p, ownersz);
+            slen = strlen(owner);
+	    replacenonprintable(owner, slen, '.');
 	  }
 	}
       } 
