@@ -86,7 +86,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc,v 1.25 2003/09/15 07:28:45 fyodor Exp $ */
+/* $Id: output.cc,v 1.28 2003/12/18 21:42:56 fyodor Exp $ */
 
 #include "output.h"
 #include "osscan.h"
@@ -143,10 +143,12 @@ static int getServiceXMLBuf(struct serviceDeductions *sd, char *xmlbuf,
   } else rpcbuf[0] = '\0';
 
   snprintf(xmlbuf, xmlbuflen, 
-	   "<service name=\"%s\"%s %smethod=\"%s\" conf=\"%d\"%s />", sd->name, 
-	   versionxmlstring.c_str(), 
-	   (sd->dtype == SERVICE_DETECTION_TABLE)? "table" : "probed", 
+	   "<service name=\"%s\"%s %smethod=\"%s\" conf=\"%d\"%s />", 
+	   sd->name,
+	   versionxmlstring.c_str(),
 	   (sd->service_tunnel == SERVICE_TUNNEL_SSL)? "tunnel=\"ssl\" " : "",
+	   (sd->dtype == SERVICE_DETECTION_TABLE)? "table" : "probed", 
+
 	   sd->name_confidence, rpcbuf);
 
   return 0;
@@ -201,6 +203,9 @@ void printportoutput(Target *currenths, PortList *plist) {
   char rpcmachineinfo[64];
   char portinfo[64];
   char xmlbuf[512];
+  char grepvers[256];
+  char grepown[64];
+  char *p;
   char *state;
   char serviceinfo[64];
   char *name=NULL;
@@ -305,7 +310,6 @@ void printportoutput(Target *currenths, PortList *plist) {
 	proto = nmap_getprotbynum(htons(current->portno));
 	snprintf(portinfo, sizeof(portinfo), "%-24s",
 		 proto?proto->p_name: "unknown");
-	rowno++;
 	Tbl->addItemFormatted(rowno, portcol, "%d", portno);
 	Tbl->addItem(rowno, statecol, true, state);
 	Tbl->addItem(rowno, servicecol, true, portinfo);
@@ -378,9 +382,39 @@ void printportoutput(Target *currenths, PortList *plist) {
 	if (*sd.fullversion)
 	  Tbl->addItem(rowno, versioncol, true, sd.fullversion);
 
-	log_write(LOG_MACHINE,"%d/%s/%s/%s/%s/%s//", current->portno, state, 
-		  protocol, (current->owner)? current->owner : "",
-		  (sd.name)? sd.name: "", rpcmachineinfo);    
+	// How should we escape illegal chars in grepable output?
+	// Well, a reasonably clean way would be backslash escapes
+	// such as \/ and \\ .  // But that makes it harder to pick
+	// out fields with awk, cut, and such.  So I'm gonna use the
+	// ugly hat (fitting to grepable output) or replacing the '/'
+	// character with '|' in the version and owner fields.
+	Strncpy(grepvers, sd.fullversion, 
+		sizeof(grepvers) / sizeof(*grepvers));
+	p = grepvers;
+	while((p = strchr(p, '/'))) {
+	  *p = '|';
+	  p++;
+	}
+	if (!current->owner) *grepown = '\0';
+	else {
+	  Strncpy(grepown, current->owner, 
+		  sizeof(grepown) / sizeof(*grepown));
+	  p = grepown;
+	  while((p = strchr(p, '/'))) {
+	    *p = '|';
+	    p++;
+	  }
+	}
+	if (!sd.name) serviceinfo[0] = '\0';
+	else {
+	  p = serviceinfo;
+	  while((p = strchr(p, '/'))) {
+	    *p = '|';
+	    p++;
+	  }
+	}
+	log_write(LOG_MACHINE,"%d/%s/%s/%s/%s/%s/%s/", current->portno, state, 
+		  protocol, grepown, serviceinfo, rpcmachineinfo, grepvers);    
 	
 	log_write(LOG_XML, "<port protocol=\"%s\" portid=\"%d\">", protocol, current->portno);
 	log_write(LOG_XML, "<state state=\"%s\" />", state);
@@ -816,7 +850,7 @@ static void printosclassificationoutput(const struct OS_Classification_Results *
       if (OSR->OSC[classno]->OS_Generation) {
 	snprintf(tmpbuf, sizeof(tmpbuf), " osgen=\"%s\"", OSR->OSC[classno]->OS_Generation);
       } else tmpbuf[0] = '\0';
-      log_write(LOG_XML, "<osclass type=\"%s\" vendor=\"%s\" osfamily=\"%s\"%s accuracy=\"%d\"/>\n", OSR->OSC[classno]->Device_Type, OSR->OSC[classno]->OS_Vendor, OSR->OSC[classno]->OS_Family, tmpbuf, (int) (OSR->OSC_Accuracy[classno] * 100));
+      log_write(LOG_XML, "<osclass type=\"%s\" vendor=\"%s\" osfamily=\"%s\"%s accuracy=\"%d\" />\n", OSR->OSC[classno]->Device_Type, OSR->OSC[classno]->OS_Vendor, OSR->OSC[classno]->OS_Family, tmpbuf, (int) (OSR->OSC_Accuracy[classno] * 100));
     }
 
     // Now to create the fodder for normal output
@@ -1022,7 +1056,7 @@ void printosscanoutput(Target *currenths) {
 	 sprintf(p, "%hX", currenths->seq.ipids[i]);
 	 while(*p) p++;
        }
-       log_write(LOG_XML, "<ipidsequence class=\"%s\" values=\"%s\"/>\n", ipidclass2ascii(currenths->seq.ipid_seqclass), numlst);
+       log_write(LOG_XML, "<ipidsequence class=\"%s\" values=\"%s\" />\n", ipidclass2ascii(currenths->seq.ipid_seqclass), numlst);
        if (o.verbose)
 	 log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,"IPID Sequence Generation: %s\n", ipidclass2ascii(currenths->seq.ipid_seqclass));
        log_write(LOG_MACHINE,"\tIPID Seq: %s", ipidclass2ascii(currenths->seq.ipid_seqclass));
@@ -1040,7 +1074,7 @@ void printosscanoutput(Target *currenths) {
        if (currenths->seq.ts_seqclass != TS_SEQ_UNSUPPORTED) {
 	 log_write(LOG_XML, " values=\"%s\"", numlst);
        }
-       log_write(LOG_XML, "/>\n");
+       log_write(LOG_XML, " />\n");
      }
   }
 }
